"""App configuration for the django_colors app."""

from django.apps import AppConfig


class DjangoColorsConfig(AppConfig):
    """Configuration class for the django_colors app."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "django_colors"
    verbose_name = "Django Colors"
